// Datei AbstrakterMethodenTest.java
/* ------------------------------------------------------------------------
Diese abstrakte Klasse eignet sich als Oberklasse fuer JUnit-Programme,
die (Klassen-) Methoden testen und reflektiv auf diese Methoden zugreifen.
---------------------------------------------------------------------------
Bekannte Erweiterungen dieser Klasse:
   AnzahlVerschiedeneJut
   DurchschnittJut
   EnthaeltJut
   EnthaeltGleicheJut
   VereinigungJut
------------------------------------------------------------------------ */
import junit.framework.Test;      // Eine Schnittstelle
import junit.framework.TestCase;  // Eine Test-Klasse
import junit.framework.TestSuite; // Eine Test-Klasse

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;

public abstract class AbstrakterMethodenTest extends TestCase {
   // ---------------------------------------------------------------------
   final String     klassenName;
   final String     methodenName;
   final Class<?>[] paramTypen;
   final Class<?>   klasse;
   final Method     methode;

   final String     signatur;
   final String     mldKeineKlasse;
   final String     mldKeineMethode;
   // ---------------------------------------------------------------------
   public AbstrakterMethodenTest(
      String      klassenName,
      String      methodenName,
      Class<?>... paramTypen)
   {
      this. klassenName =  klassenName;
      this.methodenName = methodenName;
      this.paramTypen   = paramTypen;

      StringBuilder sig = new StringBuilder(methodenName + "(");
      for (int i=0; i<paramTypen.length-1; i++) {
         Class<?> typ = paramTypen[i];
         sig.append(getTypeName(typ) + ", ");
      }
      if (paramTypen.length>0) {
         Class<?> typ = paramTypen[paramTypen.length-1];
         sig.append(getTypeName(typ));
      }
      sig.append(")");
      signatur = sig.toString();

      mldKeineKlasse = String.format("Eine Klasse namens %s scheint " +
         "(noch?) nicht implementiert zu sein!%n", klassenName);
      mldKeineMethode = String.format("Die Klasse %s scheint keine "  +
         "ffentliche Methode%nmit der Signatur %s zu enthalten!%n",
         klassenName, signatur);
      try {
         this.klasse       = Class.forName(klassenName);
         this.methode      = klasse.getMethod(methodenName, paramTypen);
      } catch (ClassNotFoundException ex) {
         printf(mldKeineKlasse);
         System.exit(1);
         throw new Error(ex);
      } catch (NoSuchMethodException ex) {
         printf(mldKeineMethode);
         System.exit(1);
         throw new Error(ex);
      } catch (Exception ex) {
         throw new Error(ex);
      } // try/catch
   } // Konstruktor AbstrakterMethodenTest
   // ---------------------------------------------------------------------
   protected void checkOneCall(Object sollErg, Object... param) {
      // Ruft die zu testende Methode M einmal auf und prueft, ob
      // ihr Ergebnis gleich dem sollErg ist (mit dem JUnit-Befehl
      // assertEquals).
      try {
         Object  istErg  = methode.invoke(null, param);
         String  istErgS = ToString.sts( istErg);
         String sollErgS = ToString.sts(sollErg);
         String meldung  = ToString.uts(param);
         meldung         = meldung.substring(1, meldung.length()-1);
         meldung         = methodenName + "(" + meldung + ")";

         assertEquals(meldung, sollErgS, istErgS);

      } catch (Exception ex) {
         throw new Error(ex);
      }
   } // checkOneCall
   // ---------------------------------------------------------------------
   public static String getTypeName(Class<?> kob) {
      // Liefert den Namen des durch kob repraesentierten Typs in
      // lesbarer Form. Beispiele:
      // +------------------------+----------------------+---------+
      // | Von kob.getName()      | Von dieser Methode   | Stufig- |
      // | gelieferter Typname:   | gelieferter Typname: | keit:   |
      // +------------------------+----------------------+---------+
      // | int                    | int                  |  -      |
      // | [I                     | int[]                |  1      |
      // | boolean                | boolean              |  -      |
      // | [[boolean              | boolean[][]          |  2      |
      // | java.lang.String       | String               |  -      |
      // | [Ljava.lang.String;    | String[]             |  1      |
      // | [[[Ljava.lang.String;  | String[][][]         |  3      |
      // +------------------------+----------------------+---------+

      StringBuilder erg         = new StringBuilder();
      String        name        = kob.getName();
      int           anzKlammern = 0;
      int           ind         = 0;

      // Anzahl der '['-Zeichen am Anfang von Name ermitteln
      // (d.h. die Stufigkeit eines Reihungstyps):
      for (; ind<name.length(); ind++) {
         if (name.charAt(ind) != '[') break;
         anzKlammern++;
      }

      // Die '['-Zeichen (0 oder mehr) abschneiden:
      name = name.substring(ind);

      // Wenn name jetzt nur noch auch einem Zeichen besteht, ist
      // das vermutlich der 1-Zeichen-Code eines primitiven Typs:
      if (name.length()==1) {
         char c = name.charAt(0);
         switch (c) {
            case 'B': erg.append("byte");    break;
            case 'C': erg.append("char");    break;
            case 'S': erg.append("short");   break;
            case 'I': erg.append("int");     break;
            case 'J': erg.append("long");    break;
            case 'F': erg.append("float");   break;
            case 'D': erg.append("double");  break;
            case 'Z': erg.append("boolean"); break;
            // c enthaelt nicht den 1-Zeichen-Codenamen eines primitiven
            // Typs, sondern (hoffentlich) den 1-Zeichen-langen Namen
            // einer Klasse (sehr ungewoehnlich aber moeglich):
            default : erg.append(c);
         }
      } else if (name.startsWith("L") && name.endsWith(";")) {
         // Wenn name z.B. gleich "Ljava.lang.String;" ist:
         int von = Math.max(name.lastIndexOf('.')+1, 1);
         int bis = name.length()-1;
         erg.append(name.substring(von, bis));
      } else {
         // Wenn name z.B. gleich "java.lang.String" oder gleich "int" ist:
         int von = Math.max(name.lastIndexOf('.')+1, 0);
         erg.append(name.substring(von));
      }

      // Fuer jede Stufe eines Reihungstyps "[]" an erg anhaengen:
      while (anzKlammern-- > 0) erg.append("[]");
      return erg.toString();
   } // getTypeName
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      printf("AbstrakterMethodenTest: Diese Klasse ist ABSTRAKT!%n");
      printf("-------------------------------------------------%n");
      printf("Bekannte konkrete Erweiterungen dieser Klasse:   %n");
      printf("AnzahlVerschiedeneJut, DurchschnitJut,           %n");
      printf("EnthaeltGleicheJut, EnthaeltJut, VereinigungJut. %n");
      printf("-------------------------------------------------%n");
   } // main
   // ---------------------------------------------------------------------
} // class AbstrakterMethodenTest
